#!/bin/sh

# \
  if (test -z "$WISH"); then export WISH=wish; fi
# \
  exec "$WISH" "$0" -name wish ${1+"$@"}

# ======================================================================

# example3.tcl --

# Timer example.
# Create a stopwatch timer which reaches the maximum value.

# Version   : 0.3.0
# Author    : Mark G. Saye
# Email     : markgsaye@yahoo.com
# Copyright : Copyright (C) 2003
# Date      : December 09, 2003

# See the file "LICENSE.txt" or "LICENSE.html" for information on usage
# and distribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# ======================================================================

proc alarm {} {

  set t [clock seconds]
  set time [clock format $t -format "%H:%M:%S %Z"]
  set date [clock format $t -format "%a %b %d %Y"]
  puts "Countdown finished at $time on $date"
  tk_messageBox \
    -icon info \
    -message "Countdown finished at\n$time\non\n$date" \
    -title "Alarm" \
    -type ok

}

# ======================================================================

proc refresh {timer} {

  upvar #0 $timer _

  set h $_(hours)
  set m $_(minutes)
  set s $_(seconds)
  set u $_(milliseconds)

  set d [expr {$h / 24}]
  set h [expr {$h % 24}]

  set _(time) [format \
    "%12s days %2d hours %2d minutes %2d.%03d seconds" $d $h $m $s $u]

}

# ======================================================================

  package require timer

# Set maximum time in milliseconds - check for 64-bit support
  if { [expr 0xffffffff] == -1 } {
    # max 32-bit integer = 2147483647
    set max 0x7fffffff
  } else {
    # max 64-bit integer = 9223372036854775807
    set max 0x7fffffffffffffff
  }
  set reset [expr {$max - 5000}]

  set timer [timer::create \
    -reset $reset \
    -alarm  [list alarm] \
    -update [list refresh] \
  ]

  entry  .value -textvariable ${timer}(time) -width 52
  refresh $timer

  button .start -text "Start/Stop" \
    -command [list timer::startstop $timer]

  button .reset -text "Reset" \
    -command [list timer::reset     $timer]

  pack .value -side top -expand 1 -fill x
  pack .reset .start -side right

# ======================================================================

