#!/bin/sh

# \
  if (test -z "$WISH"); then export WISH=wish; fi
# \
  exec "$WISH" "$0" -name wish ${1+"$@"}

# ======================================================================

# watch.tcl --

# A stopwatch using the timer package.

# Version   : 0.3.0
# Author    : Mark G. Saye
# Email     : markgsaye@yahoo.com
# Copyright : Copyright (C) 2003
# Date      : December 09, 2003

# See the file "LICENSE.txt" or "LICENSE.html" for information on usage
# and distribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# ======================================================================

  namespace eval watch {}

# ======================================================================

proc watch::pkginit {} {

# ----------------------------------------------------------------------

  variable watch

  variable debug ; if { ![info exists debug] } { set debug 1 }

  debug 2 "proc watch::pkginit\n"

# ----------------------------------------------------------------------

  if { [catch {file normalize [info script]} scr] } {
    set list {}
    foreach item [file split [file join [pwd] [info script]]] {
      switch -- $item {
        "."     {}
        ".."    { set list [lreplace $list end end] }
        default { lappend list $item }
      }
    }
    set scr [eval file join $list]
    unset list
  }

  set dir [file dirname $scr]

  debug 3 "  script = \[$scr\]\n"

# ----------------------------------------------------------------------

  variable pkginfo ; array set pkginfo [list \
    package      watch \
    version      0.3.0 \
    script       $scr \
    directory    $dir \
    name        "Watch" \
    description "A stopwatch." \
    copyright   "Copyright (C) 2003" \
    author      "Mark G. Saye" \
    email       "markgsaye@yahoo.com" \
    date        "December 09, 2003" \
    title       "Watch" \
    iconname    "Watch" \
    iconbitmap  [file nativename [file join $dir watch14.xbm]] \
  ]

# ----------------------------------------------------------------------

  if { [catch {package require timer} ver] } {
    lappend ::auto_path [file dirname $dir]
    set ver [package require timer]
  }

  set packages {
  }

  foreach package $packages {
    debug 3 "  package require $package\n"
    set ver [eval package require $package]
    debug 3 "  package require $package ... $ver\n"
  }

# ----------------------------------------------------------------------

  package provide $pkginfo(package) $pkginfo(version)

# ----------------------------------------------------------------------

}

# ======================================================================

proc watch::close {W} {

  debug 2 "proc watch::close \[$W\]\n"

# ----------------------------------------------------------------------

  variable $W
  upvar  0 $W _

# ----------------------------------------------------------------------

  catch {unset _}
  catch {destroy $W}

# ----------------------------------------------------------------------

  foreach child [winfo children .] {
    if { [string equal [winfo toplevel $child] $child] } {
      return
    }
  }
  exit 0

# ----------------------------------------------------------------------

}

# ======================================================================

proc watch::create {W} {

  debug 2 "proc watch::create \[$W\]\n"

# ----------------------------------------------------------------------

  set nc [namespace current]

  variable debug
  variable watch

  variable $W
  upvar  0 $W _

# ----------------------------------------------------------------------

  set rowi -1

  grid columnconfigure $W 0 -weight 1
  grid columnconfigure $W 1 -weight 1
  grid columnconfigure $W 2 -weight 1

# ----------------------------------------------------------------------

# Create a timer for this window
  set timer [timer::create \
    -debug $debug \
    -interval 1 \
    -stop   [list ${nc}::stop $W] \
  ]

  set _(timer) $timer
  reset $W

# ----------------------------------------------------------------------

  set font [font create \
    -family courier \
    -size 20 \
    -weight bold \
  ]

  set entry $W.time
  entry $entry \
    -font $font \
    -textvariable ${timer}(time) \
    -relief sunken \
    -bd 1

  incr rowi
  grid $entry -column 0 -row $rowi -sticky new -columnspan 3

# ----------------------------------------------------------------------

  set coli -1
  incr rowi
  foreach item [list lap reset startstop] {
    set button $W.$item
    button $button \
      -text "[string totitle $item]" \
      -width 5 \
      -command [list ${nc}::$item $W]

    incr coli
    grid $button -column $coli -row $rowi -sticky ew
  }

  $W.startstop configure \
    -activebackground green \
    -bg green \
    -text "Start"

  set _(running) 0

# ----------------------------------------------------------------------

  set frame $W.laptitle
  frame $frame -relief flat -borderwidth 0

  incr rowi
  grid $frame -column 0 -row $rowi -sticky new -columnspan 3

  set label $frame.label
  label $label -text "Lap Times"
  pack  $label -side left -expand 0 -fill none -anchor e

  foreach {item proc} {
    clear  lap:clear
    delete lap:delete
    new    spawn
  } {
    set button $frame.$item
    button $button \
      -text "[string totitle $item]" \
      -width 6 \
      -padx 0 -pady 0 \
      -command [list ${nc}::$proc $W]
    pack $button -side right -expand 0 -fill none -anchor e
  }

# ----------------------------------------------------------------------

  set scroll $W.laps
  set listbox $scroll.listbox

  frame $scroll -relief sunken -bd 1

  listbox $listbox \
    -relief flat \
    -bd 0 \
    -selectmode extended \
    -yscrollcommand [list $scroll.y set]

  scrollbar $scroll.y -orient vertical -command [list $listbox yview]

  grid columnconfigure $scroll 0 -weight 1

  grid $listbox  -column 0 -row 0 -sticky nsew
  grid $scroll.y -column 1 -row 0 -sticky nsew

  set _(listbox) $listbox

  set _(listbox,data) {}

  $listbox see end

  incr rowi
  grid    rowconfigure $W $rowi -weight 1
  grid $scroll -column 0 -columnspan 3 -row $rowi -sticky nsew

# ----------------------------------------------------------------------

}

# ======================================================================

proc watch::debug {level string} {

  variable debug

  set i [uplevel [list info level]]

  if { $level <= $debug } {
    puts -nonewline stderr "\[$i\] watch: $string"
  }

}

# ======================================================================

proc watch::lap {W} {

  debug 2 "proc watch::lap \[$W\]\n"

# ----------------------------------------------------------------------

  variable $W
  upvar  0 $W _

  set timer $_(timer)
  upvar #0 $timer state

# ----------------------------------------------------------------------

  set listbox $_(listbox)
  set scroll [winfo parent $listbox]
  set y $scroll.y
  set get [$y get]
  set sy  [lindex $get end]

  set now $state(now)

  set string $state(time)

# ----------------------------------------------------------------------

  if { [$listbox size] } {
    set last [lindex $_(listbox,data) end]
    set time [expr {$now - $last}]
    if { $time < 0 } {
      set time [expr {0 - $time}]
      set - -
    } else {
      set - +
    }

    set t [expr {$time / 1000}]
    set h [expr {$t    / 3600}]
    set t [expr {$t    % 3600}]
    set m [expr {$t    /   60}]
    set s [expr {$t    %   60}]
    set u [expr {$time % 1000}]

    set H [format %02d $h]
    set M [format %02d $m]
    set S [format %02d $s]
    set U [format %03d $u]
    append string " ${-}$H:$M:$S.$U"
  }

# ----------------------------------------------------------------------

  $listbox insert end $string
  lappend _(listbox,data) $now

# ----------------------------------------------------------------------

  if { ([llength $get] == 4) || ($sy == "1.0") } {
    $listbox see end
  }

# ----------------------------------------------------------------------

}

# ======================================================================

proc watch::lap:clear {W} {

  debug 2 "proc watch::lap:delete \[$W\]\n"

# ----------------------------------------------------------------------

  variable $W
  upvar  0 $W _

# ----------------------------------------------------------------------

  set listbox $_(listbox)

  $listbox delete 0 end

  set _(listbox,data) {}

# ----------------------------------------------------------------------

}

# ======================================================================

proc watch::lap:delete {W} {

  debug 2 "proc watch::lap:delete \[$W\]\n"

# ----------------------------------------------------------------------

  variable $W
  upvar  0 $W _

  upvar 0 _(listbox,data) data

# ----------------------------------------------------------------------

  set listbox $_(listbox)
  set selection [$listbox curselection]

  if { $selection == {} } {
    return
  } else {
    foreach index [lsort -integer -decreasing $selection] {
      set data [lreplace $data $index $index]
      $listbox delete $index
    }
  }

# ----------------------------------------------------------------------

}

# ======================================================================

proc watch::main {argc argv} {

  debug 2 "proc watch::main \[$argc\] \[$argv\]\n"

# ----------------------------------------------------------------------

  array set data [eval parse $argv]

# ----------------------------------------------------------------------

  wm withdraw .

# ----------------------------------------------------------------------

  eval spawn $data(-args)

# ----------------------------------------------------------------------

  return -1

}

# ======================================================================

proc watch::parse {args} {

  debug 2 "proc watch::parse \[$args\]\n"

# ----------------------------------------------------------------------

  variable debug
  variable pkginfo
  variable watch

# ----------------------------------------------------------------------

  set data(-args) {}

# ----------------------------------------------------------------------

  set argc [llength $args]

  for {set argi 0} {$argi < $argc} {incr argi} {

    set arg [lindex $args $argi]

    switch  -- $arg {
      -h -
      --help {
        puts stdout "$pkginfo(name) $pkginfo(version)"
        puts stdout "$pkginfo(copyright), $pkginfo(author)"
        puts stdout ""
        puts stdout "$pkginfo(description)"
        puts stdout ""

        set script [file tail $pkginfo(script)]

        puts stdout "Usage: $script \[OPTION(s)\] \[FILE(s)\]"
        puts stdout ""
        puts stdout "where OPTION(s) can be :-"
        puts stdout ""
        puts stdout "  -d, --debug     : display debug information."
        puts stdout "  -h, --help      : display this help screen."
        puts stdout "  -V, --version   : display version information."
        puts stdout ""
        return -code return 0
      }
      -D -
      --debug { incr debug }
      -V -
      --version {
        puts stdout "$pkginfo(name) $pkginfo(version)"
        return -code return 0
      }
      default { lappend data(-args) $arg }
    }
  }

# ----------------------------------------------------------------------

  return [array get data]

}

# ======================================================================

proc watch::reset {W} {

  debug 2 "proc watch::reset \[$W\]\n"

# ----------------------------------------------------------------------

  variable $W
  upvar  0 $W _

  set timer $_(timer)
  upvar #0 $timer state

# ----------------------------------------------------------------------

  timer::reset $timer 0

# ----------------------------------------------------------------------

}

# ======================================================================

proc watch::spawn {args} {

  debug 2 "proc watch::spawn \[$args\]\n"

# ----------------------------------------------------------------------

  variable pkginfo

# ----------------------------------------------------------------------

  array set data [eval parse $args]

# ----------------------------------------------------------------------

  for {set i 1} {[winfo exists .watch-$i]} {incr i} {}
  set W .watch-$i
  toplevel $W

  wm iconbitmap $W @$pkginfo(iconbitmap)
  wm iconname   $W  $pkginfo(iconname)
  wm protocol   $W  WM_DELETE_WINDOW [list watch::close $W]
  wm title      $W  $pkginfo(title)

# ----------------------------------------------------------------------

  create $W

# ----------------------------------------------------------------------

  return $W

}

# ======================================================================

proc watch::start {W} {

  debug 2 "proc watch::start \[$W\]\n"

# ----------------------------------------------------------------------

  variable $W
  upvar  0 $W _

  set timer $_(timer)
  upvar #0 $timer state

# ----------------------------------------------------------------------

  after 0 timer::start $timer

# ----------------------------------------------------------------------

  $W.startstop configure \
    -bg red \
    -activebackground red \
    -text "Stop"

# ----------------------------------------------------------------------

  debug 2 "proc watch::start \[$W\] -> Done.\n"

}

# ======================================================================

proc watch::startstop {W} {

  debug 2 "proc watch::startstop \[$W\]\n"

# ----------------------------------------------------------------------

  variable $W
  upvar  0 $W _

  set timer $_(timer)
  upvar #0 $timer state

# ----------------------------------------------------------------------

  if { $state(running) } {
    watch::stop  $W
  } else {
    watch::start $W
  }

# ----------------------------------------------------------------------

  debug 2 "proc watch::startstop \[$W\] -> Done.\n"

}

# ======================================================================

proc watch::stop {W} {

  debug 2 "proc watch::stop \[$W\]\n"

# ----------------------------------------------------------------------

  variable $W
  upvar  0 $W _

  set timer $_(timer)
  upvar #0 $timer state

# ----------------------------------------------------------------------

  after 0 timer::stop $timer

# ----------------------------------------------------------------------

  $W.startstop configure \
    -bg green \
    -activebackground green \
    -text "Start"

# ----------------------------------------------------------------------

}

# ======================================================================

proc watch::update {W} {

  debug 2 "proc watch::update \[$W\]\n"

# ----------------------------------------------------------------------

  variable $W
  upvar  0 $W _

  set timer $_(timer)
  upvar #0 $timer state

# ----------------------------------------------------------------------

  set H [format %04d $state(hours)]
  set M [format %02d $state(minutes)]
  set S [format %02d $state(seconds)]
  set U [format %03d $state(milliseconds)]

  set _(time) "$H:$M:$S.$U"

# ----------------------------------------------------------------------

}

# ======================================================================

  if { [info exists argv0] && [string equal [info script] $argv0] } {
    catch {uplevel #0 source $::tcl_rcFileName}
    watch::pkginit
    set code [watch::main $argc $argv]
    if { $code > -1 } { exit $code }
  } else {
    watch::pkginit
  }

  return watch::main

# ======================================================================

